/*
    Bloboats - a boat racing game by Blobtrox
    Copyright (C) 2006  Markus "MakeGho" Kettunen

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License along
    with this program; if not, write to the Free Software Foundation, Inc.,
    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
*/

#ifndef _PLAYER_H_
#define _PLAYER_H_

#include <string.h>
#include <stdio.h>
#include <stdlib.h>

#define NICK_MAXLEN 10
#define MAXLEVELS 30

#include "SDL.h"

class player {
public:
	player();
	~player();



	bool setnick(char *name);

	char *name;
	double times[MAXLEVELS][10];

	bool inuse;

	Uint32 tuxfound;
};



#endif

